@@ -2,6 +2,7 @@
 # kbhirombhakdi@stsci.edu
 
 from rescalex.rescale import Rescale
+from outlierdetection.outlierdetection import outlierdetection
 import numpy as np
 import copy
 
@@ -12,22 +13,28 @@ class Polynomial2D:
     ### Polynomial2D().data = dict of data. These data are inputs, and will not be modified.
     ### Polynomial2D().model = dict of data associated to a specified 2D polynomial model.
     ### Polynomial2D().rescale = dict of specification for rescaling X1,X2,Y. 
-    ###    This can be specified by Polynomial2D(rescale=(a=bool,b,c,d)) where a = bool to specify whether rescaling would be performed.
+    ###    This can be specified by Polynomial2D(rescale=(a,b,c,d)) where a = bool to specify whether rescaling would be performed.
     ###    For b,c,d, each is a dict specifying arguments for rescalex.rescale.Rescale for X1,X2,Y respectively.
     ###    For example, b = {'method'='linear','minmax'=(-1.,1.),'params':'None'}.
     ###    Set b = None for not performing rescale with X1, and vice versa.
+    ### Polynomial2D().sigclip = dict of specification for sigma-cipping.
+    ###    This can be specified by Polynomial2D(sigclip=(bool,niter,sigma)) where 
+    ###       bool to specify whether sigma-clipping would be performed.
+    ###       niter = numbers of iteration
+    ###       sigma = sigma clipping level
     ### Polynomial2D().compute() = compute YFIT given a model and data.
     ### Polynomial2D().fit() = fit for 2D polynomial coefficients given a model and data. This method includes an interation routine for outlier detection using outlierdetection.py.
     ### Polynomial2D().test() = randomly simulate x1, x2, and coefficients given nsize and norder.
     ### Polynomial2D().data['MASK'] keeps the original mask of data with False as good data.
-    ### Polynomial2D().model['MASKFIT'] keeps the latest mask from iterative fitting data and rejecting outliers.
+    ### Polynomial2D().model['MASKFIT'] updated mask from iterative fitting with sigclip 
     Note: if rescale is performed, Polynomial2D().model['COEF'] results from rescaled space
     while Polynomial2D().model['YFIT'] results in non-rescaled spcae.
     #########################################
-    ### call: obj = Polynomial2D(x1,x2,y,mask,norder,coef,rescale)
+    ### call: obj = Polynomial2D(x1,x2,y,mask,norder,coef,rescale,sigclip)
     ### call: obj.data['key'] for key = 'X1','X2','Y','MASK'
     ### call: obj.model['key'] for key = 'NORDER','COEF',...(created using model)...
     ### call: obj.rescale for examining rescaling results
+    ### call: obj.sigclip for examining sigma-clipping setting
     x1,x2,y,mask must have the same dimensions.
     mask, if not specified, is set to all False (= good data).
     norder is non-negative integer representing the highest polynomial order.
@@ -48,7 +55,8 @@ class Polynomial2D:
     """
     def __init__(self,x1=None,x2=None,y=None,mask=None,
                  norder=None,coef=None,
-                 rescale=(False,None,None,None)
+                 rescale=(False,None,None,None),
+                 sigclip=(False,None,None)
                 ):
         if (mask is None) and (x1 is not None):
                 mask = np.full_like(x1,False,dtype=bool)
@@ -67,6 +75,7 @@ class Polynomial2D:
                         'X2':None,
                         'Y':None
                        }
+        self.sigclip = sigclip
         self._rescale()
     ##########
     ##########
@@ -107,12 +116,30 @@ class Polynomial2D:
             newobj.model = copy.deepcopy(self.model)
         else:
             newobj = copy.deepcopy(self)
-        newobj._curvefit()
-        newobj.compute()
+        # sigclip
+        doclip,niter,sigma_level = self.sigclip
+        sentinel = True
+        counter = 0
+        while sentinel:
+            counter += 1
+            newobj._curvefit()
+            newobj.compute()
+            print('Sigma clipping = {0}, sigma level = {1}, iter #{2}'.format(doclip,sigma_level,counter))
+            if not doclip or counter>=niter:
+                break
+            else:
+                newobj._sigclip(sigma_level)
         newobj.model['YFIT'] = self.rescale['Y'].invtransform(newobj.model['YFIT'])
         self.model = copy.deepcopy(newobj.model)
     ##########
     ##########
+    def _sigclip(self,sigma_level):
+        tmp = self.data['Y'] - self.model['YFIT']
+        std = np.std(tmp)
+        n_sig = np.abs(tmp)/std
+        tmpmask = (n_sig > sigma_level) # good data == False
+        self.model['MASKFIT'] = (self.model['MASKFIT'] | tmpmask)
+        print('Update maskfit')
     def _rescale(self):
         rescale = self.rescale['RESCALE']
         x1,x2,y = self.data['X1'].copy(),self.data['X2'].copy(),self.data['Y'].copy()
